Package: SummarizedExperiment
Title: A container (S4 class) for matrix-like assays
Description: The SummarizedExperiment container contains one or more assays,
	each represented by a matrix-like object of numeric or other mode.
	The rows typically represent genomic ranges of interest and the columns
	represent samples.
biocViews: Genetics, Infrastructure, Sequencing, Annotation, Coverage,
        GenomeAnnotation
URL: https://bioconductor.org/packages/SummarizedExperiment
BugReports: https://github.com/Bioconductor/SummarizedExperiment/issues
Version: 1.41.0
License: Artistic-2.0
Encoding: UTF-8
Authors@R: c(
	person("Martin", "Morgan", role="aut"),
	person("Valerie", "Obenchain", role="aut"),
	person("Jim", "Hester", role="aut"),
	person("Hervé", "Pagès", role=c("aut", "cre"),
		email="hpages.on.github@gmail.com"))
Depends: R (>= 4.0.0), methods, MatrixGenerics (>= 1.1.3),
        GenomicRanges (>= 1.61.4), Biobase
Imports: utils, stats, tools, Matrix, BiocGenerics (>= 0.51.3),
        S4Vectors (>= 0.33.7), IRanges (>= 2.23.9), Seqinfo, S4Arrays
        (>= 1.1.1), DelayedArray (>= 0.31.12)
Suggests: GenomeInfoDb (>= 1.45.5), rhdf5, HDF5Array (>= 1.7.5),
        annotate, AnnotationDbi, GenomicFeatures, SparseArray,
        SingleCellExperiment, TxDb.Hsapiens.UCSC.hg19.knownGene,
        hgu95av2.db, airway (>= 1.15.1), BiocStyle, knitr, rmarkdown,
        RUnit, testthat, digest
VignetteBuilder: knitr
Collate: Assays-class.R SummarizedExperiment-class.R
        RangedSummarizedExperiment-class.R intra-range-methods.R
        inter-range-methods.R coverage-methods.R combine-methods.R
        findOverlaps-methods.R nearest-methods.R
        makeSummarizedExperimentFromExpressionSet.R
        makeSummarizedExperimentFromDataFrame.R
        makeSummarizedExperimentFromLoom.R zzz.R
git_url: https://git.bioconductor.org/packages/SummarizedExperiment
git_branch: devel
git_last_commit: 7a9787c
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-11-01 00:31:04 UTC; biocbuild
Author: Martin Morgan [aut],
  Valerie Obenchain [aut],
  Jim Hester [aut],
  Hervé Pagès [aut, cre]
Maintainer: Hervé Pagès <hpages.on.github@gmail.com>
